"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.initializeTemplates = exports.npmPullTemplate = void 0;

var _child_process = require("child_process");

var _unarchive = require("./unarchive");

var _helpers = require("./helpers");

var _path = require("path");

var _fsExtra = require("fs-extra");

// @ts-check
const templateDirectories = ['template', 'template-for-libs', 'template-max'];
/**
 * @param {string} depName
 * @param {string} cwd
 * @param {string} targetDir
 */

const npmPullTemplate = async (depName, cwd, targetDir) => {
  const infoProcessResult = (0, _child_process.spawnSync)('npm', ['info', depName], {
    encoding: 'utf8',
    shell: process.platform === 'win32',
    cwd
  });

  if (infoProcessResult.status !== 0) {
    throw new Error(`Couldn't find '${depName}' package in npm repository`);
  }

  const templateFullDirs = templateDirectories.map(dir => (0, _path.join)(targetDir, dir));
  const templatePathExist = await Promise.all(templateFullDirs.map(dir => (0, _fsExtra.pathExists)(dir)));
  const existingTemplatePath = templatePathExist.indexOf(true);

  if (existingTemplatePath >= 0) {
    throw new Error(`Path already exists: ${templateFullDirs[existingTemplatePath]}, it would be unsafe to pull templates to this location, please remove or backup the directory manually`);
  }

  const packResult = (0, _child_process.spawnSync)('npm', ['pack', depName], {
    encoding: 'utf8',
    shell: process.platform === 'win32',
    cwd
  });
  const packagePath = packResult.stdout.trim();
  const fullPackagePath = (0, _path.join)(cwd, packagePath);
  await (0, _unarchive.unarchiveTarGz)(fullPackagePath, cwd, {
    map: header => {
      const name = header.name.replace('package/', '');
      return { ...header,
        name
      };
    },
    ignore: (_name, headers) => {
      if (!headers) {
        return false;
      }

      return templateDirectories.every(dir => !headers.name.startsWith(dir));
    }
  });
  await (0, _fsExtra.unlink)(fullPackagePath);
  return (0, _path.relative)(cwd, (0, _helpers.resolveTemplatesDir)(templateFullDirs));
};
/**
 * @param {string} template
 * @param {string} cwd
 * @param {string} targetDir
 * @returns {Promise<TemplateInfo>}
 */


exports.npmPullTemplate = npmPullTemplate;

const pullTemplates = async (template, cwd, targetDir) => {
  const templatePath = (0, _path.isAbsolute)(template) ? template : (0, _path.join)(cwd, template);
  const templatesStat = await (0, _fsExtra.stat)(templatePath).catch(_err => null);

  if (templatesStat && templatesStat.isDirectory()) {
    return {
      type: 'directory',
      dir: templatePath
    };
  }

  const result = await npmPullTemplate(template, cwd, targetDir);
  return {
    type: 'package',
    dir: (0, _path.join)(cwd, result)
  };
};
/**
 * @param {string | undefined} template
 * @param {string} cwd
 * @param {string} targetDir
 * @returns {Promise<TemplateInfo[]>}
 */


const initializeTemplates = async (template, cwd, targetDir) => {
  /**
   * @type TemplateInfo
   */
  const defTemplateInfo = {
    type: 'default',
    dir: (0, _helpers.resolveTemplatesDir)()
  };
  const templateInfo = template ? await pullTemplates(template, cwd, targetDir) : defTemplateInfo;
  const templates = templateInfo.type !== 'default' ? [defTemplateInfo, templateInfo] : [defTemplateInfo];
  return templates;
};

exports.initializeTemplates = initializeTemplates;
//# sourceMappingURL=pullTemplate.js.map