"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.initCli = initCli;
exports.initCliModule = exports.init = void 0;

var _child_process = require("child_process");

var _fsExtra = require("fs-extra");

var _path = require("path");

var _patch = require("./patch");

var _helpers = require("../helpers");

var _yargs = _interopRequireDefault(require("yargs"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// @ts-check
const PKG_JSON = 'package.json';
/**
 * @param {string} initDir
 */

const npmInitIfRequired = async initDir => {
  const packageJsonPath = (0, _path.resolve)(initDir, PKG_JSON);
  const exists = await (0, _fsExtra.pathExists)(packageJsonPath);

  if (exists) {
    return false;
  }

  const initProcessResult = (0, _child_process.spawnSync)('npm', ['init'], {
    stdio: 'inherit',
    encoding: 'utf8',
    shell: process.platform === 'win32',
    cwd: initDir
  });

  if (initProcessResult.status) {
    process.exit(initProcessResult.status);
  }

  if (!(0, _fsExtra.existsSync)(packageJsonPath)) {
    process.exit(0);
  }

  return true;
};
/**
 * @param {{ templateDirs: string[], initDir: string }} param0
 */


const copyFromTemplates = async ({
  templateDirs,
  initDir
}) => {
  for (const templateDir of templateDirs) {
    const toCopyDir = (0, _path.join)(templateDir, 'to-copy');
    const dirToCopy = await (0, _fsExtra.stat)(toCopyDir).catch(() => Promise.resolve(null));

    if (!dirToCopy || !dirToCopy.isDirectory()) {
      throw new Error('Source template directory doesnt exist: ' + toCopyDir);
    }

    await (0, _fsExtra.copy)(toCopyDir, initDir, {
      filter: async (_src, dest) => {
        const destStats = await (0, _fsExtra.stat)(dest).catch(() => Promise.resolve(null));

        if (destStats && destStats.isDirectory()) {
          return true;
        }

        if (dest !== initDir) {
          console.log('init: writing', dest);
        }

        return true;
      },
      overwrite: true
    });
  }

  const dirsToCreate = [//
  'src', 'src/__tests__', 'src/__integration-tests__'];
  await Promise.all(dirsToCreate.map(dir => (0, _path.join)(initDir, dir)).map(dir => (0, _fsExtra.ensureDir)(dir).catch(() => Promise.resolve())));
};
/**
 * @param {string} cwd
 * @param {boolean | undefined} force
 */


const errorIfNotEmpty = async (cwd, force) => {
  if (force) {
    return;
  } // a minimum set of files that are safe


  const ignoreFiles = ['package.json', 'yarn.lock', 'package-lock.json', 'node_modules'];
  const currentDirectoryContents = await (0, _fsExtra.readdir)(cwd);
  const dirContentsFiltered = currentDirectoryContents.filter(item => !item.startsWith('.') && !ignoreFiles.includes(item));

  if (dirContentsFiltered.length > 0) {
    if (!currentDirectoryContents.includes('.git')) {
      console.log('🚨  Oops. It seems that current directory contains files other than package.json but git is not initialized. ' + 'The init command overwrites files, so it is recommended to initialize a git repository for backup.');
      throw new Error('Current directory not quite empty');
    }

    const untrackedFilesCmd = (0, _child_process.spawnSync)('git', ['ls-files', '--other', '--directory', '--exclude-standard'], {
      cwd,
      encoding: 'utf8',
      shell: process.platform === 'win32'
    });

    if (untrackedFilesCmd.stdout && untrackedFilesCmd.stdout.trim().length > 0) {
      console.log('🚨  Oops. It seems that current directory contains untracked files. ' + 'The init command overwrites files, so it is recommended to at least stage current changes in git before initializing.');
      throw new Error('Current directory not quite empty');
    }
  }
};
/**
 * @param {InitParams} param0
 */


const init = async ({
  cwd = process.cwd(),
  targetDirectory = process.cwd(),
  template,
  force
} = {}) => {
  const currentDir = (0, _path.resolve)(cwd);
  const initDir = (0, _path.resolve)(targetDirectory);
  const currentDirStat = await (0, _fsExtra.stat)(currentDir).catch(_err => null);

  if (!currentDirStat || !currentDirStat.isDirectory()) {
    throw new Error('Current directory must resolve to an existing directory');
  }

  const initDirStat = await (0, _fsExtra.stat)(initDir).catch(_err => null);

  if (!initDirStat || !initDirStat.isDirectory()) {
    throw new Error('Init directory must resolve to an existing directory');
  }

  await errorIfNotEmpty(initDir, force);
  const templates = await (0, _helpers.initializeTemplates)(template, currentDir, initDir);
  await npmInitIfRequired(initDir);

  const copyTemplates = () => copyFromTemplates({
    templateDirs: templates.map(item => item.dir),
    initDir
  });

  const generate = () => (0, _patch.patch)({
    initializedTemplates: templates,
    cwd: currentDir,
    targetDirectory: initDir,
    forceOverwrites: true,
    aggressive: true
  });

  await copyTemplates();
  await generate();

  if (template) {
    for (const tmplt of templates) {
      if (tmplt.type === 'package') {
        await (0, _fsExtra.remove)(tmplt.dir);
      }
    }
  }
};
/**
 * @param {import('yargs').Arguments<{ directory?: string, force: boolean, template?: string }>} args
 */


exports.init = init;

async function initHandler(args) {
  try {
    await init({ ...(args.directory && {
        targetDirectory: args.directory
      }),
      ...(args.template && {
        template: args.template
      }),
      ...(typeof args.force === 'boolean' && {
        force: args.force
      })
    });
  } catch (err) {
    console.error('💥  ', err);
    process.exit(1);
  }
}

const initCliModule = {
  command: ['init'],

  /**
   * @param {import('yargs').Argv} y
   */
  builder: y => y.option('directory', {
    string: true,
    description: 'Directory to initialize new package in, defaults to process.cwd()'
  }).option('template', {
    string: true,
    description: 'Name of the package to use as template, or path to a template directory'
  }).option('force', {
    boolean: true,
    default: false,
    description: 'Disable non-empty directory warning'
  }),

  /**
   * @param {import('yargs').Arguments<{ force: boolean, template?: string }>} args
   */
  handler: initHandler,
  describe: 'Initialize new package, or integrate with existing package - the cli will overwrite existing files'
};
exports.initCliModule = initCliModule;

async function initCli() {
  const args = initCliModule.builder(_yargs.default).parse();
  await initHandler(args);
}
//# sourceMappingURL=init.js.map